#include <QtWaylandClient/private/qwayland-xdg-shell.h>

QT_BEGIN_NAMESPACE

namespace QtWayland {
    xdg_shell::xdg_shell(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    xdg_shell::xdg_shell(struct ::xdg_shell *obj)
        : m_xdg_shell(obj)
    {
        init_listener();
    }

    xdg_shell::xdg_shell()
        : m_xdg_shell(0)
    {
    }

    xdg_shell::~xdg_shell()
    {
    }

    void xdg_shell::init(struct ::wl_registry *registry, int id, int version)
    {
        m_xdg_shell = static_cast<struct ::xdg_shell *>(wl_registry_bind(registry, id, &xdg_shell_interface, version));
        init_listener();
    }

    void xdg_shell::init(struct ::xdg_shell *obj)
    {
        m_xdg_shell = obj;
        init_listener();
    }

    bool xdg_shell::isInitialized() const
    {
        return m_xdg_shell != 0;
    }

    void xdg_shell::use_unstable_version(int32_t version)
    {
        xdg_shell_use_unstable_version(
            m_xdg_shell,
            version);
    }

    struct ::xdg_surface *xdg_shell::get_xdg_surface(struct ::wl_surface *surface)
    {
        return xdg_shell_get_xdg_surface(
            m_xdg_shell,
            surface);
    }

    struct ::xdg_popup *xdg_shell::get_xdg_popup(struct ::wl_surface *surface, struct ::wl_surface *parent, struct ::wl_seat *seat, uint32_t serial, int32_t x, int32_t y, uint32_t flags)
    {
        return xdg_shell_get_xdg_popup(
            m_xdg_shell,
            surface,
            parent,
            seat,
            serial,
            x,
            y,
            flags);
    }

    void xdg_shell::pong(uint32_t serial)
    {
        xdg_shell_pong(
            m_xdg_shell,
            serial);
    }

    void xdg_shell::xdg_shell_ping(uint32_t )
    {
    }

    void xdg_shell::handle_ping(
        void *data,
        struct ::xdg_shell *object,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<xdg_shell *>(data)->xdg_shell_ping(
            serial);
    }

    const struct xdg_shell_listener xdg_shell::m_xdg_shell_listener = {
        xdg_shell::handle_ping
    };

    void xdg_shell::init_listener()
    {
        xdg_shell_add_listener(m_xdg_shell, &m_xdg_shell_listener, this);
    }

    xdg_surface::xdg_surface(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    xdg_surface::xdg_surface(struct ::xdg_surface *obj)
        : m_xdg_surface(obj)
    {
        init_listener();
    }

    xdg_surface::xdg_surface()
        : m_xdg_surface(0)
    {
    }

    xdg_surface::~xdg_surface()
    {
    }

    void xdg_surface::init(struct ::wl_registry *registry, int id, int version)
    {
        m_xdg_surface = static_cast<struct ::xdg_surface *>(wl_registry_bind(registry, id, &xdg_surface_interface, version));
        init_listener();
    }

    void xdg_surface::init(struct ::xdg_surface *obj)
    {
        m_xdg_surface = obj;
        init_listener();
    }

    bool xdg_surface::isInitialized() const
    {
        return m_xdg_surface != 0;
    }

    void xdg_surface::destroy()
    {
        xdg_surface_destroy(
            m_xdg_surface);
    }

    void xdg_surface::set_transient_for(struct ::wl_surface *parent)
    {
        xdg_surface_set_transient_for(
            m_xdg_surface,
            parent);
    }

    void xdg_surface::set_margin(int32_t left_margin, int32_t right_margin, int32_t top_margin, int32_t bottom_margin)
    {
        xdg_surface_set_margin(
            m_xdg_surface,
            left_margin,
            right_margin,
            top_margin,
            bottom_margin);
    }

    void xdg_surface::set_title(const QString &title)
    {
        xdg_surface_set_title(
            m_xdg_surface,
            title.toUtf8().constData());
    }

    void xdg_surface::set_app_id(const QString &app_id)
    {
        xdg_surface_set_app_id(
            m_xdg_surface,
            app_id.toUtf8().constData());
    }

    void xdg_surface::move(struct ::wl_seat *seat, uint32_t serial)
    {
        xdg_surface_move(
            m_xdg_surface,
            seat,
            serial);
    }

    void xdg_surface::resize(struct ::wl_seat *seat, uint32_t serial, uint32_t edges)
    {
        xdg_surface_resize(
            m_xdg_surface,
            seat,
            serial,
            edges);
    }

    void xdg_surface::set_output(struct ::wl_output *output)
    {
        xdg_surface_set_output(
            m_xdg_surface,
            output);
    }

    void xdg_surface::request_change_state(uint32_t state_type, uint32_t value, uint32_t serial)
    {
        xdg_surface_request_change_state(
            m_xdg_surface,
            state_type,
            value,
            serial);
    }

    void xdg_surface::ack_change_state(uint32_t state_type, uint32_t value, uint32_t serial)
    {
        xdg_surface_ack_change_state(
            m_xdg_surface,
            state_type,
            value,
            serial);
    }

    void xdg_surface::set_minimized()
    {
        xdg_surface_set_minimized(
            m_xdg_surface);
    }

    void xdg_surface::xdg_surface_configure(int32_t , int32_t )
    {
    }

    void xdg_surface::handle_configure(
        void *data,
        struct ::xdg_surface *object,
        int32_t width,
        int32_t height)
    {
        Q_UNUSED(object);
        static_cast<xdg_surface *>(data)->xdg_surface_configure(
            width,
            height);
    }

    void xdg_surface::xdg_surface_change_state(uint32_t , uint32_t , uint32_t )
    {
    }

    void xdg_surface::handle_change_state(
        void *data,
        struct ::xdg_surface *object,
        uint32_t state_type,
        uint32_t value,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<xdg_surface *>(data)->xdg_surface_change_state(
            state_type,
            value,
            serial);
    }

    void xdg_surface::xdg_surface_activated()
    {
    }

    void xdg_surface::handle_activated(
        void *data,
        struct ::xdg_surface *object)
    {
        Q_UNUSED(object);
        static_cast<xdg_surface *>(data)->xdg_surface_activated();
    }

    void xdg_surface::xdg_surface_deactivated()
    {
    }

    void xdg_surface::handle_deactivated(
        void *data,
        struct ::xdg_surface *object)
    {
        Q_UNUSED(object);
        static_cast<xdg_surface *>(data)->xdg_surface_deactivated();
    }

    void xdg_surface::xdg_surface_close()
    {
    }

    void xdg_surface::handle_close(
        void *data,
        struct ::xdg_surface *object)
    {
        Q_UNUSED(object);
        static_cast<xdg_surface *>(data)->xdg_surface_close();
    }

    const struct xdg_surface_listener xdg_surface::m_xdg_surface_listener = {
        xdg_surface::handle_configure,
        xdg_surface::handle_change_state,
        xdg_surface::handle_activated,
        xdg_surface::handle_deactivated,
        xdg_surface::handle_close
    };

    void xdg_surface::init_listener()
    {
        xdg_surface_add_listener(m_xdg_surface, &m_xdg_surface_listener, this);
    }

    xdg_popup::xdg_popup(struct ::wl_registry *registry, int id, int version)
    {
        init(registry, id, version);
    }

    xdg_popup::xdg_popup(struct ::xdg_popup *obj)
        : m_xdg_popup(obj)
    {
        init_listener();
    }

    xdg_popup::xdg_popup()
        : m_xdg_popup(0)
    {
    }

    xdg_popup::~xdg_popup()
    {
    }

    void xdg_popup::init(struct ::wl_registry *registry, int id, int version)
    {
        m_xdg_popup = static_cast<struct ::xdg_popup *>(wl_registry_bind(registry, id, &xdg_popup_interface, version));
        init_listener();
    }

    void xdg_popup::init(struct ::xdg_popup *obj)
    {
        m_xdg_popup = obj;
        init_listener();
    }

    bool xdg_popup::isInitialized() const
    {
        return m_xdg_popup != 0;
    }

    void xdg_popup::destroy()
    {
        xdg_popup_destroy(
            m_xdg_popup);
    }

    void xdg_popup::xdg_popup_popup_done(uint32_t )
    {
    }

    void xdg_popup::handle_popup_done(
        void *data,
        struct ::xdg_popup *object,
        uint32_t serial)
    {
        Q_UNUSED(object);
        static_cast<xdg_popup *>(data)->xdg_popup_popup_done(
            serial);
    }

    const struct xdg_popup_listener xdg_popup::m_xdg_popup_listener = {
        xdg_popup::handle_popup_done
    };

    void xdg_popup::init_listener()
    {
        xdg_popup_add_listener(m_xdg_popup, &m_xdg_popup_listener, this);
    }
}

QT_END_NAMESPACE
