#include <QtCompositor/private/qwayland-server-hardware-integration.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_hardware_integration::qt_hardware_integration(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_hardware_integration::qt_hardware_integration(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_hardware_integration::qt_hardware_integration()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_hardware_integration::~qt_hardware_integration()
    {
    }

    void qt_hardware_integration::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_hardware_integration::Resource *qt_hardware_integration::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_hardware_integration::Resource *qt_hardware_integration::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_hardware_integration::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_hardware_integration_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_hardware_integration::Resource *qt_hardware_integration::hardware_integration_allocate()
    {
        return new Resource;
    }

    void qt_hardware_integration::hardware_integration_bind_resource(Resource *)
    {
    }

    void qt_hardware_integration::hardware_integration_destroy_resource(Resource *)
    {
    }

    void qt_hardware_integration::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_hardware_integration *that = static_cast<qt_hardware_integration *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_hardware_integration::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_hardware_integration *that = resource->hardware_integration_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->hardware_integration_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_hardware_integration::Resource *qt_hardware_integration::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = hardware_integration_allocate();
        resource->hardware_integration_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_hardware_integration_interface, version, id);
        wl_resource_set_implementation(handle, 0, resource, destroy_func);
        resource->handle = handle;
        hardware_integration_bind_resource(resource);
        return resource;
    }

    void qt_hardware_integration::send_client_backend(const QString &name)
    {
        send_client_backend(
            m_resource->handle,
            name);
    }

    void qt_hardware_integration::send_client_backend(struct ::wl_resource *resource, const QString &name)
    {
        qt_hardware_integration_send_client_backend(
            resource,
            name.toUtf8().constData());
    }


    void qt_hardware_integration::send_server_backend(const QString &name)
    {
        send_server_backend(
            m_resource->handle,
            name);
    }

    void qt_hardware_integration::send_server_backend(struct ::wl_resource *resource, const QString &name)
    {
        qt_hardware_integration_send_server_backend(
            resource,
            name.toUtf8().constData());
    }

}

QT_END_NAMESPACE
