#include <QtCompositor/private/qwayland-server-input-method.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    wl_input_method_context::wl_input_method_context(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    wl_input_method_context::wl_input_method_context(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    wl_input_method_context::wl_input_method_context()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    wl_input_method_context::~wl_input_method_context()
    {
    }

    void wl_input_method_context::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    wl_input_method_context::Resource *wl_input_method_context::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    wl_input_method_context::Resource *wl_input_method_context::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void wl_input_method_context::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::wl_input_method_context_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    wl_input_method_context::Resource *wl_input_method_context::input_method_context_allocate()
    {
        return new Resource;
    }

    void wl_input_method_context::input_method_context_bind_resource(Resource *)
    {
    }

    void wl_input_method_context::input_method_context_destroy_resource(Resource *)
    {
    }

    void wl_input_method_context::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        wl_input_method_context *that = static_cast<wl_input_method_context *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void wl_input_method_context::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        wl_input_method_context *that = resource->input_method_context_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->input_method_context_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    wl_input_method_context::Resource *wl_input_method_context::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = input_method_context_allocate();
        resource->input_method_context_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::wl_input_method_context_interface, version, id);
        wl_resource_set_implementation(handle, &m_wl_input_method_context_interface, resource, destroy_func);
        resource->handle = handle;
        input_method_context_bind_resource(resource);
        return resource;
    }

    const struct ::wl_input_method_context_interface wl_input_method_context::m_wl_input_method_context_interface = {
        wl_input_method_context::handle_destroy,
        wl_input_method_context::handle_commit_string,
        wl_input_method_context::handle_preedit_string,
        wl_input_method_context::handle_preedit_styling,
        wl_input_method_context::handle_preedit_cursor,
        wl_input_method_context::handle_delete_surrounding_text,
        wl_input_method_context::handle_cursor_position,
        wl_input_method_context::handle_modifiers_map,
        wl_input_method_context::handle_keysym,
        wl_input_method_context::handle_grab_keyboard,
        wl_input_method_context::handle_key,
        wl_input_method_context::handle_modifiers,
        wl_input_method_context::handle_language,
        wl_input_method_context::handle_text_direction
    };

    void wl_input_method_context::input_method_context_destroy(Resource *)
    {
    }

    void wl_input_method_context::input_method_context_commit_string(Resource *, uint32_t , const QString &)
    {
    }

    void wl_input_method_context::input_method_context_preedit_string(Resource *, uint32_t , const QString &, const QString &)
    {
    }

    void wl_input_method_context::input_method_context_preedit_styling(Resource *, uint32_t , uint32_t , uint32_t )
    {
    }

    void wl_input_method_context::input_method_context_preedit_cursor(Resource *, int32_t )
    {
    }

    void wl_input_method_context::input_method_context_delete_surrounding_text(Resource *, int32_t , uint32_t )
    {
    }

    void wl_input_method_context::input_method_context_cursor_position(Resource *, int32_t , int32_t )
    {
    }

    void wl_input_method_context::input_method_context_modifiers_map(Resource *, wl_array *)
    {
    }

    void wl_input_method_context::input_method_context_keysym(Resource *, uint32_t , uint32_t , uint32_t , uint32_t , uint32_t )
    {
    }

    void wl_input_method_context::input_method_context_grab_keyboard(Resource *, uint32_t)
    {
    }

    void wl_input_method_context::input_method_context_key(Resource *, uint32_t , uint32_t , uint32_t , uint32_t )
    {
    }

    void wl_input_method_context::input_method_context_modifiers(Resource *, uint32_t , uint32_t , uint32_t , uint32_t , uint32_t )
    {
    }

    void wl_input_method_context::input_method_context_language(Resource *, uint32_t , const QString &)
    {
    }

    void wl_input_method_context::input_method_context_text_direction(Resource *, uint32_t , uint32_t )
    {
    }


    void wl_input_method_context::handle_destroy(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_destroy(
            r);
    }

    void wl_input_method_context::handle_commit_string(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial,
        const char *text)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_commit_string(
            r,
            serial,
            QString::fromUtf8(text));
    }

    void wl_input_method_context::handle_preedit_string(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial,
        const char *text,
        const char *commit)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_preedit_string(
            r,
            serial,
            QString::fromUtf8(text),
            QString::fromUtf8(commit));
    }

    void wl_input_method_context::handle_preedit_styling(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t index,
        uint32_t length,
        uint32_t style)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_preedit_styling(
            r,
            index,
            length,
            style);
    }

    void wl_input_method_context::handle_preedit_cursor(
        ::wl_client *client,
        struct wl_resource *resource,
        int32_t index)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_preedit_cursor(
            r,
            index);
    }

    void wl_input_method_context::handle_delete_surrounding_text(
        ::wl_client *client,
        struct wl_resource *resource,
        int32_t index,
        uint32_t length)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_delete_surrounding_text(
            r,
            index,
            length);
    }

    void wl_input_method_context::handle_cursor_position(
        ::wl_client *client,
        struct wl_resource *resource,
        int32_t index,
        int32_t anchor)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_cursor_position(
            r,
            index,
            anchor);
    }

    void wl_input_method_context::handle_modifiers_map(
        ::wl_client *client,
        struct wl_resource *resource,
        wl_array *map)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_modifiers_map(
            r,
            map);
    }

    void wl_input_method_context::handle_keysym(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial,
        uint32_t time,
        uint32_t sym,
        uint32_t state,
        uint32_t modifiers)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_keysym(
            r,
            serial,
            time,
            sym,
            state,
            modifiers);
    }

    void wl_input_method_context::handle_grab_keyboard(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t keyboard)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_grab_keyboard(
            r,
            keyboard);
    }

    void wl_input_method_context::handle_key(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial,
        uint32_t time,
        uint32_t key,
        uint32_t state)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_key(
            r,
            serial,
            time,
            key,
            state);
    }

    void wl_input_method_context::handle_modifiers(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial,
        uint32_t mods_depressed,
        uint32_t mods_latched,
        uint32_t mods_locked,
        uint32_t group)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_modifiers(
            r,
            serial,
            mods_depressed,
            mods_latched,
            mods_locked,
            group);
    }

    void wl_input_method_context::handle_language(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial,
        const char *language)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_language(
            r,
            serial,
            QString::fromUtf8(language));
    }

    void wl_input_method_context::handle_text_direction(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial,
        uint32_t direction)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_method_context *>(r->input_method_context_object)->input_method_context_text_direction(
            r,
            serial,
            direction);
    }

    void wl_input_method_context::send_surrounding_text(const QString &text, uint32_t cursor, uint32_t anchor)
    {
        send_surrounding_text(
            m_resource->handle,
            text,
            cursor,
            anchor);
    }

    void wl_input_method_context::send_surrounding_text(struct ::wl_resource *resource, const QString &text, uint32_t cursor, uint32_t anchor)
    {
        wl_input_method_context_send_surrounding_text(
            resource,
            text.toUtf8().constData(),
            cursor,
            anchor);
    }


    void wl_input_method_context::send_reset()
    {
        send_reset(
            m_resource->handle);
    }

    void wl_input_method_context::send_reset(struct ::wl_resource *resource)
    {
        wl_input_method_context_send_reset(
            resource);
    }


    void wl_input_method_context::send_content_type(uint32_t hint, uint32_t purpose)
    {
        send_content_type(
            m_resource->handle,
            hint,
            purpose);
    }

    void wl_input_method_context::send_content_type(struct ::wl_resource *resource, uint32_t hint, uint32_t purpose)
    {
        wl_input_method_context_send_content_type(
            resource,
            hint,
            purpose);
    }


    void wl_input_method_context::send_invoke_action(uint32_t button, uint32_t index)
    {
        send_invoke_action(
            m_resource->handle,
            button,
            index);
    }

    void wl_input_method_context::send_invoke_action(struct ::wl_resource *resource, uint32_t button, uint32_t index)
    {
        wl_input_method_context_send_invoke_action(
            resource,
            button,
            index);
    }


    void wl_input_method_context::send_commit_state(uint32_t serial)
    {
        send_commit_state(
            m_resource->handle,
            serial);
    }

    void wl_input_method_context::send_commit_state(struct ::wl_resource *resource, uint32_t serial)
    {
        wl_input_method_context_send_commit_state(
            resource,
            serial);
    }


    void wl_input_method_context::send_preferred_language(const QString &language)
    {
        send_preferred_language(
            m_resource->handle,
            language);
    }

    void wl_input_method_context::send_preferred_language(struct ::wl_resource *resource, const QString &language)
    {
        wl_input_method_context_send_preferred_language(
            resource,
            language.toUtf8().constData());
    }


    wl_input_method::wl_input_method(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    wl_input_method::wl_input_method(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    wl_input_method::wl_input_method()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    wl_input_method::~wl_input_method()
    {
    }

    void wl_input_method::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    wl_input_method::Resource *wl_input_method::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    wl_input_method::Resource *wl_input_method::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void wl_input_method::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::wl_input_method_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    wl_input_method::Resource *wl_input_method::input_method_allocate()
    {
        return new Resource;
    }

    void wl_input_method::input_method_bind_resource(Resource *)
    {
    }

    void wl_input_method::input_method_destroy_resource(Resource *)
    {
    }

    void wl_input_method::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        wl_input_method *that = static_cast<wl_input_method *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void wl_input_method::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        wl_input_method *that = resource->input_method_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->input_method_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    wl_input_method::Resource *wl_input_method::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = input_method_allocate();
        resource->input_method_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::wl_input_method_interface, version, id);
        wl_resource_set_implementation(handle, 0, resource, destroy_func);
        resource->handle = handle;
        input_method_bind_resource(resource);
        return resource;
    }

    void wl_input_method::send_activate(struct ::wl_resource *id)
    {
        send_activate(
            m_resource->handle,
            id);
    }

    void wl_input_method::send_activate(struct ::wl_resource *resource, struct ::wl_resource *id)
    {
        wl_input_method_send_activate(
            resource,
            id);
    }


    void wl_input_method::send_deactivate(struct ::wl_resource *context)
    {
        send_deactivate(
            m_resource->handle,
            context);
    }

    void wl_input_method::send_deactivate(struct ::wl_resource *resource, struct ::wl_resource *context)
    {
        wl_input_method_send_deactivate(
            resource,
            context);
    }


    wl_input_panel::wl_input_panel(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    wl_input_panel::wl_input_panel(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    wl_input_panel::wl_input_panel()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    wl_input_panel::~wl_input_panel()
    {
    }

    void wl_input_panel::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    wl_input_panel::Resource *wl_input_panel::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    wl_input_panel::Resource *wl_input_panel::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void wl_input_panel::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::wl_input_panel_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    wl_input_panel::Resource *wl_input_panel::input_panel_allocate()
    {
        return new Resource;
    }

    void wl_input_panel::input_panel_bind_resource(Resource *)
    {
    }

    void wl_input_panel::input_panel_destroy_resource(Resource *)
    {
    }

    void wl_input_panel::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        wl_input_panel *that = static_cast<wl_input_panel *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void wl_input_panel::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        wl_input_panel *that = resource->input_panel_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->input_panel_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    wl_input_panel::Resource *wl_input_panel::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = input_panel_allocate();
        resource->input_panel_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::wl_input_panel_interface, version, id);
        wl_resource_set_implementation(handle, &m_wl_input_panel_interface, resource, destroy_func);
        resource->handle = handle;
        input_panel_bind_resource(resource);
        return resource;
    }

    const struct ::wl_input_panel_interface wl_input_panel::m_wl_input_panel_interface = {
        wl_input_panel::handle_get_input_panel_surface
    };

    void wl_input_panel::input_panel_get_input_panel_surface(Resource *, uint32_t, struct ::wl_resource *)
    {
    }


    void wl_input_panel::handle_get_input_panel_surface(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t id,
        struct ::wl_resource *surface)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_panel *>(r->input_panel_object)->input_panel_get_input_panel_surface(
            r,
            id,
            surface);
    }

    wl_input_panel_surface::wl_input_panel_surface(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    wl_input_panel_surface::wl_input_panel_surface(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    wl_input_panel_surface::wl_input_panel_surface()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    wl_input_panel_surface::~wl_input_panel_surface()
    {
    }

    void wl_input_panel_surface::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    wl_input_panel_surface::Resource *wl_input_panel_surface::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    wl_input_panel_surface::Resource *wl_input_panel_surface::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void wl_input_panel_surface::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::wl_input_panel_surface_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    wl_input_panel_surface::Resource *wl_input_panel_surface::input_panel_surface_allocate()
    {
        return new Resource;
    }

    void wl_input_panel_surface::input_panel_surface_bind_resource(Resource *)
    {
    }

    void wl_input_panel_surface::input_panel_surface_destroy_resource(Resource *)
    {
    }

    void wl_input_panel_surface::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        wl_input_panel_surface *that = static_cast<wl_input_panel_surface *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void wl_input_panel_surface::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        wl_input_panel_surface *that = resource->input_panel_surface_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->input_panel_surface_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    wl_input_panel_surface::Resource *wl_input_panel_surface::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = input_panel_surface_allocate();
        resource->input_panel_surface_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::wl_input_panel_surface_interface, version, id);
        wl_resource_set_implementation(handle, &m_wl_input_panel_surface_interface, resource, destroy_func);
        resource->handle = handle;
        input_panel_surface_bind_resource(resource);
        return resource;
    }

    const struct ::wl_input_panel_surface_interface wl_input_panel_surface::m_wl_input_panel_surface_interface = {
        wl_input_panel_surface::handle_set_toplevel,
        wl_input_panel_surface::handle_set_overlay_panel
    };

    void wl_input_panel_surface::input_panel_surface_set_toplevel(Resource *, struct ::wl_resource *, uint32_t )
    {
    }

    void wl_input_panel_surface::input_panel_surface_set_overlay_panel(Resource *)
    {
    }


    void wl_input_panel_surface::handle_set_toplevel(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *output,
        uint32_t position)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_panel_surface *>(r->input_panel_surface_object)->input_panel_surface_set_toplevel(
            r,
            output,
            position);
    }

    void wl_input_panel_surface::handle_set_overlay_panel(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_input_panel_surface *>(r->input_panel_surface_object)->input_panel_surface_set_overlay_panel(
            r);
    }
}

QT_END_NAMESPACE
