#include <QtCompositor/private/qwayland-server-output-extension.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_output_extension::qt_output_extension(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_output_extension::qt_output_extension(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_output_extension::qt_output_extension()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_output_extension::~qt_output_extension()
    {
    }

    void qt_output_extension::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_output_extension::Resource *qt_output_extension::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_output_extension::Resource *qt_output_extension::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_output_extension::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_output_extension_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_output_extension::Resource *qt_output_extension::output_extension_allocate()
    {
        return new Resource;
    }

    void qt_output_extension::output_extension_bind_resource(Resource *)
    {
    }

    void qt_output_extension::output_extension_destroy_resource(Resource *)
    {
    }

    void qt_output_extension::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_output_extension *that = static_cast<qt_output_extension *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_output_extension::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_output_extension *that = resource->output_extension_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->output_extension_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_output_extension::Resource *qt_output_extension::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = output_extension_allocate();
        resource->output_extension_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_output_extension_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_output_extension_interface, resource, destroy_func);
        resource->handle = handle;
        output_extension_bind_resource(resource);
        return resource;
    }

    const struct ::qt_output_extension_interface qt_output_extension::m_qt_output_extension_interface = {
        qt_output_extension::handle_get_extended_output
    };

    void qt_output_extension::output_extension_get_extended_output(Resource *, uint32_t, struct ::wl_resource *)
    {
    }


    void qt_output_extension::handle_get_extended_output(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t id,
        struct ::wl_resource *output)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_output_extension *>(r->output_extension_object)->output_extension_get_extended_output(
            r,
            id,
            output);
    }

    qt_extended_output::qt_extended_output(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_extended_output::qt_extended_output(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_extended_output::qt_extended_output()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_extended_output::~qt_extended_output()
    {
    }

    void qt_extended_output::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_extended_output::Resource *qt_extended_output::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_extended_output::Resource *qt_extended_output::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_extended_output::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_extended_output_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_extended_output::Resource *qt_extended_output::extended_output_allocate()
    {
        return new Resource;
    }

    void qt_extended_output::extended_output_bind_resource(Resource *)
    {
    }

    void qt_extended_output::extended_output_destroy_resource(Resource *)
    {
    }

    void qt_extended_output::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_extended_output *that = static_cast<qt_extended_output *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_extended_output::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_extended_output *that = resource->extended_output_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->extended_output_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_extended_output::Resource *qt_extended_output::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = extended_output_allocate();
        resource->extended_output_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_extended_output_interface, version, id);
        wl_resource_set_implementation(handle, 0, resource, destroy_func);
        resource->handle = handle;
        extended_output_bind_resource(resource);
        return resource;
    }
}

QT_END_NAMESPACE
