#include <QtCompositor/private/qwayland-server-qtkey-extension.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_key_extension::qt_key_extension(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_key_extension::qt_key_extension(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_key_extension::qt_key_extension()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_key_extension::~qt_key_extension()
    {
    }

    void qt_key_extension::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_key_extension::Resource *qt_key_extension::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_key_extension::Resource *qt_key_extension::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_key_extension::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_key_extension_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_key_extension::Resource *qt_key_extension::key_extension_allocate()
    {
        return new Resource;
    }

    void qt_key_extension::key_extension_bind_resource(Resource *)
    {
    }

    void qt_key_extension::key_extension_destroy_resource(Resource *)
    {
    }

    void qt_key_extension::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_key_extension *that = static_cast<qt_key_extension *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_key_extension::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_key_extension *that = resource->key_extension_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->key_extension_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_key_extension::Resource *qt_key_extension::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = key_extension_allocate();
        resource->key_extension_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_key_extension_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_key_extension_interface, resource, destroy_func);
        resource->handle = handle;
        key_extension_bind_resource(resource);
        return resource;
    }

    const struct ::qt_key_extension_interface qt_key_extension::m_qt_key_extension_interface = {
        qt_key_extension::handle_dummy
    };

    void qt_key_extension::key_extension_dummy(Resource *)
    {
    }


    void qt_key_extension::handle_dummy(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_key_extension *>(r->key_extension_object)->key_extension_dummy(
            r);
    }

    void qt_key_extension::send_qtkey(struct ::wl_resource *surface, uint32_t time, uint32_t type, uint32_t key, uint32_t modifiers, uint32_t nativeScanCode, uint32_t nativeVirtualKey, uint32_t nativeModifiers, const QString &text, uint32_t autorepeat, uint32_t count)
    {
        send_qtkey(
            m_resource->handle,
            surface,
            time,
            type,
            key,
            modifiers,
            nativeScanCode,
            nativeVirtualKey,
            nativeModifiers,
            text,
            autorepeat,
            count);
    }

    void qt_key_extension::send_qtkey(struct ::wl_resource *resource, struct ::wl_resource *surface, uint32_t time, uint32_t type, uint32_t key, uint32_t modifiers, uint32_t nativeScanCode, uint32_t nativeVirtualKey, uint32_t nativeModifiers, const QString &text, uint32_t autorepeat, uint32_t count)
    {
        qt_key_extension_send_qtkey(
            resource,
            surface,
            time,
            type,
            key,
            modifiers,
            nativeScanCode,
            nativeVirtualKey,
            nativeModifiers,
            text.toUtf8().constData(),
            autorepeat,
            count);
    }

}

QT_END_NAMESPACE
