#include <QtCompositor/private/qwayland-server-sub-surface-extension.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_sub_surface_extension::qt_sub_surface_extension(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_sub_surface_extension::qt_sub_surface_extension(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_sub_surface_extension::qt_sub_surface_extension()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_sub_surface_extension::~qt_sub_surface_extension()
    {
    }

    void qt_sub_surface_extension::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_sub_surface_extension::Resource *qt_sub_surface_extension::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_sub_surface_extension::Resource *qt_sub_surface_extension::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_sub_surface_extension::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_sub_surface_extension_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_sub_surface_extension::Resource *qt_sub_surface_extension::sub_surface_extension_allocate()
    {
        return new Resource;
    }

    void qt_sub_surface_extension::sub_surface_extension_bind_resource(Resource *)
    {
    }

    void qt_sub_surface_extension::sub_surface_extension_destroy_resource(Resource *)
    {
    }

    void qt_sub_surface_extension::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_sub_surface_extension *that = static_cast<qt_sub_surface_extension *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_sub_surface_extension::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_sub_surface_extension *that = resource->sub_surface_extension_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->sub_surface_extension_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_sub_surface_extension::Resource *qt_sub_surface_extension::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = sub_surface_extension_allocate();
        resource->sub_surface_extension_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_sub_surface_extension_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_sub_surface_extension_interface, resource, destroy_func);
        resource->handle = handle;
        sub_surface_extension_bind_resource(resource);
        return resource;
    }

    const struct ::qt_sub_surface_extension_interface qt_sub_surface_extension::m_qt_sub_surface_extension_interface = {
        qt_sub_surface_extension::handle_get_sub_surface_aware_surface
    };

    void qt_sub_surface_extension::sub_surface_extension_get_sub_surface_aware_surface(Resource *, uint32_t, struct ::wl_resource *)
    {
    }


    void qt_sub_surface_extension::handle_get_sub_surface_aware_surface(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t id,
        struct ::wl_resource *surface)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_sub_surface_extension *>(r->sub_surface_extension_object)->sub_surface_extension_get_sub_surface_aware_surface(
            r,
            id,
            surface);
    }

    qt_sub_surface::qt_sub_surface(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_sub_surface::qt_sub_surface(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_sub_surface::qt_sub_surface()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_sub_surface::~qt_sub_surface()
    {
    }

    void qt_sub_surface::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_sub_surface::Resource *qt_sub_surface::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_sub_surface::Resource *qt_sub_surface::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_sub_surface::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_sub_surface_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_sub_surface::Resource *qt_sub_surface::sub_surface_allocate()
    {
        return new Resource;
    }

    void qt_sub_surface::sub_surface_bind_resource(Resource *)
    {
    }

    void qt_sub_surface::sub_surface_destroy_resource(Resource *)
    {
    }

    void qt_sub_surface::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_sub_surface *that = static_cast<qt_sub_surface *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_sub_surface::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_sub_surface *that = resource->sub_surface_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->sub_surface_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_sub_surface::Resource *qt_sub_surface::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = sub_surface_allocate();
        resource->sub_surface_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_sub_surface_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_sub_surface_interface, resource, destroy_func);
        resource->handle = handle;
        sub_surface_bind_resource(resource);
        return resource;
    }

    const struct ::qt_sub_surface_interface qt_sub_surface::m_qt_sub_surface_interface = {
        qt_sub_surface::handle_attach_sub_surface,
        qt_sub_surface::handle_move_sub_surface,
        qt_sub_surface::handle_raise,
        qt_sub_surface::handle_lower
    };

    void qt_sub_surface::sub_surface_attach_sub_surface(Resource *, struct ::wl_resource *, int32_t , int32_t )
    {
    }

    void qt_sub_surface::sub_surface_move_sub_surface(Resource *, struct ::wl_resource *, int32_t , int32_t )
    {
    }

    void qt_sub_surface::sub_surface_raise(Resource *, struct ::wl_resource *)
    {
    }

    void qt_sub_surface::sub_surface_lower(Resource *, struct ::wl_resource *)
    {
    }


    void qt_sub_surface::handle_attach_sub_surface(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *sub_surface,
        int32_t x,
        int32_t y)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_sub_surface *>(r->sub_surface_object)->sub_surface_attach_sub_surface(
            r,
            sub_surface,
            x,
            y);
    }

    void qt_sub_surface::handle_move_sub_surface(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *sub_surface,
        int32_t x,
        int32_t y)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_sub_surface *>(r->sub_surface_object)->sub_surface_move_sub_surface(
            r,
            sub_surface,
            x,
            y);
    }

    void qt_sub_surface::handle_raise(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *sub_surface)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_sub_surface *>(r->sub_surface_object)->sub_surface_raise(
            r,
            sub_surface);
    }

    void qt_sub_surface::handle_lower(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *sub_surface)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_sub_surface *>(r->sub_surface_object)->sub_surface_lower(
            r,
            sub_surface);
    }
}

QT_END_NAMESPACE
