#include <QtCompositor/private/qwayland-server-surface-extension.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_surface_extension::qt_surface_extension(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_surface_extension::qt_surface_extension(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_surface_extension::qt_surface_extension()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_surface_extension::~qt_surface_extension()
    {
    }

    void qt_surface_extension::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_surface_extension::Resource *qt_surface_extension::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_surface_extension::Resource *qt_surface_extension::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_surface_extension::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_surface_extension_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_surface_extension::Resource *qt_surface_extension::surface_extension_allocate()
    {
        return new Resource;
    }

    void qt_surface_extension::surface_extension_bind_resource(Resource *)
    {
    }

    void qt_surface_extension::surface_extension_destroy_resource(Resource *)
    {
    }

    void qt_surface_extension::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_surface_extension *that = static_cast<qt_surface_extension *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_surface_extension::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_surface_extension *that = resource->surface_extension_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->surface_extension_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_surface_extension::Resource *qt_surface_extension::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = surface_extension_allocate();
        resource->surface_extension_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_surface_extension_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_surface_extension_interface, resource, destroy_func);
        resource->handle = handle;
        surface_extension_bind_resource(resource);
        return resource;
    }

    const struct ::qt_surface_extension_interface qt_surface_extension::m_qt_surface_extension_interface = {
        qt_surface_extension::handle_get_extended_surface
    };

    void qt_surface_extension::surface_extension_get_extended_surface(Resource *, uint32_t, struct ::wl_resource *)
    {
    }


    void qt_surface_extension::handle_get_extended_surface(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t id,
        struct ::wl_resource *surface)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_surface_extension *>(r->surface_extension_object)->surface_extension_get_extended_surface(
            r,
            id,
            surface);
    }

    qt_extended_surface::qt_extended_surface(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_extended_surface::qt_extended_surface(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_extended_surface::qt_extended_surface()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_extended_surface::~qt_extended_surface()
    {
    }

    void qt_extended_surface::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_extended_surface::Resource *qt_extended_surface::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_extended_surface::Resource *qt_extended_surface::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_extended_surface::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_extended_surface_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_extended_surface::Resource *qt_extended_surface::extended_surface_allocate()
    {
        return new Resource;
    }

    void qt_extended_surface::extended_surface_bind_resource(Resource *)
    {
    }

    void qt_extended_surface::extended_surface_destroy_resource(Resource *)
    {
    }

    void qt_extended_surface::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_extended_surface *that = static_cast<qt_extended_surface *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_extended_surface::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_extended_surface *that = resource->extended_surface_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->extended_surface_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_extended_surface::Resource *qt_extended_surface::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = extended_surface_allocate();
        resource->extended_surface_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_extended_surface_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_extended_surface_interface, resource, destroy_func);
        resource->handle = handle;
        extended_surface_bind_resource(resource);
        return resource;
    }

    const struct ::qt_extended_surface_interface qt_extended_surface::m_qt_extended_surface_interface = {
        qt_extended_surface::handle_update_generic_property,
        qt_extended_surface::handle_set_content_orientation_mask,
        qt_extended_surface::handle_set_window_flags,
        qt_extended_surface::handle_raise,
        qt_extended_surface::handle_lower
    };

    void qt_extended_surface::extended_surface_update_generic_property(Resource *, const QString &, wl_array *)
    {
    }

    void qt_extended_surface::extended_surface_set_content_orientation_mask(Resource *, int32_t )
    {
    }

    void qt_extended_surface::extended_surface_set_window_flags(Resource *, int32_t )
    {
    }

    void qt_extended_surface::extended_surface_raise(Resource *)
    {
    }

    void qt_extended_surface::extended_surface_lower(Resource *)
    {
    }


    void qt_extended_surface::handle_update_generic_property(
        ::wl_client *client,
        struct wl_resource *resource,
        const char *name,
        wl_array *value)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_extended_surface *>(r->extended_surface_object)->extended_surface_update_generic_property(
            r,
            QString::fromUtf8(name),
            value);
    }

    void qt_extended_surface::handle_set_content_orientation_mask(
        ::wl_client *client,
        struct wl_resource *resource,
        int32_t orientation)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_extended_surface *>(r->extended_surface_object)->extended_surface_set_content_orientation_mask(
            r,
            orientation);
    }

    void qt_extended_surface::handle_set_window_flags(
        ::wl_client *client,
        struct wl_resource *resource,
        int32_t flags)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_extended_surface *>(r->extended_surface_object)->extended_surface_set_window_flags(
            r,
            flags);
    }

    void qt_extended_surface::handle_raise(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_extended_surface *>(r->extended_surface_object)->extended_surface_raise(
            r);
    }

    void qt_extended_surface::handle_lower(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_extended_surface *>(r->extended_surface_object)->extended_surface_lower(
            r);
    }

    void qt_extended_surface::send_onscreen_visibility(int32_t visible)
    {
        send_onscreen_visibility(
            m_resource->handle,
            visible);
    }

    void qt_extended_surface::send_onscreen_visibility(struct ::wl_resource *resource, int32_t visible)
    {
        qt_extended_surface_send_onscreen_visibility(
            resource,
            visible);
    }


    void qt_extended_surface::send_set_generic_property(const QString &name, const QByteArray &value)
    {
        send_set_generic_property(
            m_resource->handle,
            name,
            value);
    }

    void qt_extended_surface::send_set_generic_property(struct ::wl_resource *resource, const QString &name, const QByteArray &value)
    {
        struct wl_array value_data;
        value_data.size = value.size();
        value_data.data = static_cast<void *>(const_cast<char *>(value.constData()));
        value_data.alloc = 0;

        qt_extended_surface_send_set_generic_property(
            resource,
            name.toUtf8().constData(),
            &value_data);
    }


    void qt_extended_surface::send_close()
    {
        send_close(
            m_resource->handle);
    }

    void qt_extended_surface::send_close(struct ::wl_resource *resource)
    {
        qt_extended_surface_send_close(
            resource);
    }

}

QT_END_NAMESPACE
