#include <QtCompositor/private/qwayland-server-text.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    wl_text_input::wl_text_input(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    wl_text_input::wl_text_input(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    wl_text_input::wl_text_input()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    wl_text_input::~wl_text_input()
    {
    }

    void wl_text_input::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    wl_text_input::Resource *wl_text_input::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    wl_text_input::Resource *wl_text_input::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void wl_text_input::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::wl_text_input_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    wl_text_input::Resource *wl_text_input::text_input_allocate()
    {
        return new Resource;
    }

    void wl_text_input::text_input_bind_resource(Resource *)
    {
    }

    void wl_text_input::text_input_destroy_resource(Resource *)
    {
    }

    void wl_text_input::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        wl_text_input *that = static_cast<wl_text_input *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void wl_text_input::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        wl_text_input *that = resource->text_input_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->text_input_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    wl_text_input::Resource *wl_text_input::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = text_input_allocate();
        resource->text_input_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::wl_text_input_interface, version, id);
        wl_resource_set_implementation(handle, &m_wl_text_input_interface, resource, destroy_func);
        resource->handle = handle;
        text_input_bind_resource(resource);
        return resource;
    }

    const struct ::wl_text_input_interface wl_text_input::m_wl_text_input_interface = {
        wl_text_input::handle_activate,
        wl_text_input::handle_deactivate,
        wl_text_input::handle_show_input_panel,
        wl_text_input::handle_hide_input_panel,
        wl_text_input::handle_reset,
        wl_text_input::handle_set_surrounding_text,
        wl_text_input::handle_set_content_type,
        wl_text_input::handle_set_cursor_rectangle,
        wl_text_input::handle_set_preferred_language,
        wl_text_input::handle_commit_state,
        wl_text_input::handle_invoke_action
    };

    void wl_text_input::text_input_activate(Resource *, struct ::wl_resource *, struct ::wl_resource *)
    {
    }

    void wl_text_input::text_input_deactivate(Resource *, struct ::wl_resource *)
    {
    }

    void wl_text_input::text_input_show_input_panel(Resource *)
    {
    }

    void wl_text_input::text_input_hide_input_panel(Resource *)
    {
    }

    void wl_text_input::text_input_reset(Resource *)
    {
    }

    void wl_text_input::text_input_set_surrounding_text(Resource *, const QString &, uint32_t , uint32_t )
    {
    }

    void wl_text_input::text_input_set_content_type(Resource *, uint32_t , uint32_t )
    {
    }

    void wl_text_input::text_input_set_cursor_rectangle(Resource *, int32_t , int32_t , int32_t , int32_t )
    {
    }

    void wl_text_input::text_input_set_preferred_language(Resource *, const QString &)
    {
    }

    void wl_text_input::text_input_commit_state(Resource *, uint32_t )
    {
    }

    void wl_text_input::text_input_invoke_action(Resource *, uint32_t , uint32_t )
    {
    }


    void wl_text_input::handle_activate(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *seat,
        struct ::wl_resource *surface)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_activate(
            r,
            seat,
            surface);
    }

    void wl_text_input::handle_deactivate(
        ::wl_client *client,
        struct wl_resource *resource,
        struct ::wl_resource *seat)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_deactivate(
            r,
            seat);
    }

    void wl_text_input::handle_show_input_panel(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_show_input_panel(
            r);
    }

    void wl_text_input::handle_hide_input_panel(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_hide_input_panel(
            r);
    }

    void wl_text_input::handle_reset(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_reset(
            r);
    }

    void wl_text_input::handle_set_surrounding_text(
        ::wl_client *client,
        struct wl_resource *resource,
        const char *text,
        uint32_t cursor,
        uint32_t anchor)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_set_surrounding_text(
            r,
            QString::fromUtf8(text),
            cursor,
            anchor);
    }

    void wl_text_input::handle_set_content_type(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t hint,
        uint32_t purpose)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_set_content_type(
            r,
            hint,
            purpose);
    }

    void wl_text_input::handle_set_cursor_rectangle(
        ::wl_client *client,
        struct wl_resource *resource,
        int32_t x,
        int32_t y,
        int32_t width,
        int32_t height)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_set_cursor_rectangle(
            r,
            x,
            y,
            width,
            height);
    }

    void wl_text_input::handle_set_preferred_language(
        ::wl_client *client,
        struct wl_resource *resource,
        const char *language)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_set_preferred_language(
            r,
            QString::fromUtf8(language));
    }

    void wl_text_input::handle_commit_state(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t serial)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_commit_state(
            r,
            serial);
    }

    void wl_text_input::handle_invoke_action(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t button,
        uint32_t index)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input *>(r->text_input_object)->text_input_invoke_action(
            r,
            button,
            index);
    }

    void wl_text_input::send_enter(struct ::wl_resource *surface)
    {
        send_enter(
            m_resource->handle,
            surface);
    }

    void wl_text_input::send_enter(struct ::wl_resource *resource, struct ::wl_resource *surface)
    {
        wl_text_input_send_enter(
            resource,
            surface);
    }


    void wl_text_input::send_leave()
    {
        send_leave(
            m_resource->handle);
    }

    void wl_text_input::send_leave(struct ::wl_resource *resource)
    {
        wl_text_input_send_leave(
            resource);
    }


    void wl_text_input::send_modifiers_map(const QByteArray &map)
    {
        send_modifiers_map(
            m_resource->handle,
            map);
    }

    void wl_text_input::send_modifiers_map(struct ::wl_resource *resource, const QByteArray &map)
    {
        struct wl_array map_data;
        map_data.size = map.size();
        map_data.data = static_cast<void *>(const_cast<char *>(map.constData()));
        map_data.alloc = 0;

        wl_text_input_send_modifiers_map(
            resource,
            &map_data);
    }


    void wl_text_input::send_input_panel_state(uint32_t state)
    {
        send_input_panel_state(
            m_resource->handle,
            state);
    }

    void wl_text_input::send_input_panel_state(struct ::wl_resource *resource, uint32_t state)
    {
        wl_text_input_send_input_panel_state(
            resource,
            state);
    }


    void wl_text_input::send_preedit_string(uint32_t serial, const QString &text, const QString &commit)
    {
        send_preedit_string(
            m_resource->handle,
            serial,
            text,
            commit);
    }

    void wl_text_input::send_preedit_string(struct ::wl_resource *resource, uint32_t serial, const QString &text, const QString &commit)
    {
        wl_text_input_send_preedit_string(
            resource,
            serial,
            text.toUtf8().constData(),
            commit.toUtf8().constData());
    }


    void wl_text_input::send_preedit_styling(uint32_t index, uint32_t length, uint32_t style)
    {
        send_preedit_styling(
            m_resource->handle,
            index,
            length,
            style);
    }

    void wl_text_input::send_preedit_styling(struct ::wl_resource *resource, uint32_t index, uint32_t length, uint32_t style)
    {
        wl_text_input_send_preedit_styling(
            resource,
            index,
            length,
            style);
    }


    void wl_text_input::send_preedit_cursor(int32_t index)
    {
        send_preedit_cursor(
            m_resource->handle,
            index);
    }

    void wl_text_input::send_preedit_cursor(struct ::wl_resource *resource, int32_t index)
    {
        wl_text_input_send_preedit_cursor(
            resource,
            index);
    }


    void wl_text_input::send_commit_string(uint32_t serial, const QString &text)
    {
        send_commit_string(
            m_resource->handle,
            serial,
            text);
    }

    void wl_text_input::send_commit_string(struct ::wl_resource *resource, uint32_t serial, const QString &text)
    {
        wl_text_input_send_commit_string(
            resource,
            serial,
            text.toUtf8().constData());
    }


    void wl_text_input::send_cursor_position(int32_t index, int32_t anchor)
    {
        send_cursor_position(
            m_resource->handle,
            index,
            anchor);
    }

    void wl_text_input::send_cursor_position(struct ::wl_resource *resource, int32_t index, int32_t anchor)
    {
        wl_text_input_send_cursor_position(
            resource,
            index,
            anchor);
    }


    void wl_text_input::send_delete_surrounding_text(int32_t index, uint32_t length)
    {
        send_delete_surrounding_text(
            m_resource->handle,
            index,
            length);
    }

    void wl_text_input::send_delete_surrounding_text(struct ::wl_resource *resource, int32_t index, uint32_t length)
    {
        wl_text_input_send_delete_surrounding_text(
            resource,
            index,
            length);
    }


    void wl_text_input::send_keysym(uint32_t serial, uint32_t time, uint32_t sym, uint32_t state, uint32_t modifiers)
    {
        send_keysym(
            m_resource->handle,
            serial,
            time,
            sym,
            state,
            modifiers);
    }

    void wl_text_input::send_keysym(struct ::wl_resource *resource, uint32_t serial, uint32_t time, uint32_t sym, uint32_t state, uint32_t modifiers)
    {
        wl_text_input_send_keysym(
            resource,
            serial,
            time,
            sym,
            state,
            modifiers);
    }


    void wl_text_input::send_language(uint32_t serial, const QString &language)
    {
        send_language(
            m_resource->handle,
            serial,
            language);
    }

    void wl_text_input::send_language(struct ::wl_resource *resource, uint32_t serial, const QString &language)
    {
        wl_text_input_send_language(
            resource,
            serial,
            language.toUtf8().constData());
    }


    void wl_text_input::send_text_direction(uint32_t serial, uint32_t direction)
    {
        send_text_direction(
            m_resource->handle,
            serial,
            direction);
    }

    void wl_text_input::send_text_direction(struct ::wl_resource *resource, uint32_t serial, uint32_t direction)
    {
        wl_text_input_send_text_direction(
            resource,
            serial,
            direction);
    }


    wl_text_input_manager::wl_text_input_manager(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    wl_text_input_manager::wl_text_input_manager(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    wl_text_input_manager::wl_text_input_manager()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    wl_text_input_manager::~wl_text_input_manager()
    {
    }

    void wl_text_input_manager::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    wl_text_input_manager::Resource *wl_text_input_manager::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    wl_text_input_manager::Resource *wl_text_input_manager::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void wl_text_input_manager::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::wl_text_input_manager_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    wl_text_input_manager::Resource *wl_text_input_manager::text_input_manager_allocate()
    {
        return new Resource;
    }

    void wl_text_input_manager::text_input_manager_bind_resource(Resource *)
    {
    }

    void wl_text_input_manager::text_input_manager_destroy_resource(Resource *)
    {
    }

    void wl_text_input_manager::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        wl_text_input_manager *that = static_cast<wl_text_input_manager *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void wl_text_input_manager::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        wl_text_input_manager *that = resource->text_input_manager_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->text_input_manager_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    wl_text_input_manager::Resource *wl_text_input_manager::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = text_input_manager_allocate();
        resource->text_input_manager_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::wl_text_input_manager_interface, version, id);
        wl_resource_set_implementation(handle, &m_wl_text_input_manager_interface, resource, destroy_func);
        resource->handle = handle;
        text_input_manager_bind_resource(resource);
        return resource;
    }

    const struct ::wl_text_input_manager_interface wl_text_input_manager::m_wl_text_input_manager_interface = {
        wl_text_input_manager::handle_create_text_input
    };

    void wl_text_input_manager::text_input_manager_create_text_input(Resource *, uint32_t)
    {
    }


    void wl_text_input_manager::handle_create_text_input(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t id)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<wl_text_input_manager *>(r->text_input_manager_object)->text_input_manager_create_text_input(
            r,
            id);
    }
}

QT_END_NAMESPACE
