#include <QtCompositor/private/qwayland-server-touch-extension.h>

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_touch_extension::qt_touch_extension(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_touch_extension::qt_touch_extension(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_touch_extension::qt_touch_extension()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_touch_extension::~qt_touch_extension()
    {
    }

    void qt_touch_extension::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_touch_extension::Resource *qt_touch_extension::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_touch_extension::Resource *qt_touch_extension::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_touch_extension::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_touch_extension_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_touch_extension::Resource *qt_touch_extension::touch_extension_allocate()
    {
        return new Resource;
    }

    void qt_touch_extension::touch_extension_bind_resource(Resource *)
    {
    }

    void qt_touch_extension::touch_extension_destroy_resource(Resource *)
    {
    }

    void qt_touch_extension::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_touch_extension *that = static_cast<qt_touch_extension *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_touch_extension::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_touch_extension *that = resource->touch_extension_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->touch_extension_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_touch_extension::Resource *qt_touch_extension::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = touch_extension_allocate();
        resource->touch_extension_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_touch_extension_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_touch_extension_interface, resource, destroy_func);
        resource->handle = handle;
        touch_extension_bind_resource(resource);
        return resource;
    }

    const struct ::qt_touch_extension_interface qt_touch_extension::m_qt_touch_extension_interface = {
        qt_touch_extension::handle_dummy
    };

    void qt_touch_extension::touch_extension_dummy(Resource *)
    {
    }


    void qt_touch_extension::handle_dummy(
        ::wl_client *client,
        struct wl_resource *resource)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_touch_extension *>(r->touch_extension_object)->touch_extension_dummy(
            r);
    }

    void qt_touch_extension::send_touch(uint32_t time, uint32_t id, uint32_t state, int32_t x, int32_t y, int32_t normalized_x, int32_t normalized_y, int32_t width, int32_t height, uint32_t pressure, int32_t velocity_x, int32_t velocity_y, uint32_t flags, const QByteArray &rawdata)
    {
        send_touch(
            m_resource->handle,
            time,
            id,
            state,
            x,
            y,
            normalized_x,
            normalized_y,
            width,
            height,
            pressure,
            velocity_x,
            velocity_y,
            flags,
            rawdata);
    }

    void qt_touch_extension::send_touch(struct ::wl_resource *resource, uint32_t time, uint32_t id, uint32_t state, int32_t x, int32_t y, int32_t normalized_x, int32_t normalized_y, int32_t width, int32_t height, uint32_t pressure, int32_t velocity_x, int32_t velocity_y, uint32_t flags, const QByteArray &rawdata)
    {
        struct wl_array rawdata_data;
        rawdata_data.size = rawdata.size();
        rawdata_data.data = static_cast<void *>(const_cast<char *>(rawdata.constData()));
        rawdata_data.alloc = 0;

        qt_touch_extension_send_touch(
            resource,
            time,
            id,
            state,
            x,
            y,
            normalized_x,
            normalized_y,
            width,
            height,
            pressure,
            velocity_x,
            velocity_y,
            flags,
            &rawdata_data);
    }


    void qt_touch_extension::send_configure(uint32_t flags)
    {
        send_configure(
            m_resource->handle,
            flags);
    }

    void qt_touch_extension::send_configure(struct ::wl_resource *resource, uint32_t flags)
    {
        qt_touch_extension_send_configure(
            resource,
            flags);
    }

}

QT_END_NAMESPACE
