/* 
 * Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
 * Contact: http://www.qt-project.org/legal
 * 
 * This file is part of the plugins of the Qt Toolkit.
 * 
 * $QT_BEGIN_LICENSE:BSD$
 * You may use this file under the terms of the BSD license as follows:
 * 
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
 * of its contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 * $QT_END_LICENSE$
 */

#ifndef XCOMPOSITE_CLIENT_PROTOCOL_H
#define XCOMPOSITE_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct qt_xcomposite;

extern const struct wl_interface qt_xcomposite_interface;

struct qt_xcomposite_listener {
	/**
	 * root - (none)
	 * @display_name: (none)
	 * @root_window: (none)
	 */
	void (*root)(void *data,
		     struct qt_xcomposite *qt_xcomposite,
		     const char *display_name,
		     uint32_t root_window);
};

static inline int
qt_xcomposite_add_listener(struct qt_xcomposite *qt_xcomposite,
			   const struct qt_xcomposite_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) qt_xcomposite,
				     (void (**)(void)) listener, data);
}

#define QT_XCOMPOSITE_CREATE_BUFFER	0

static inline void
qt_xcomposite_set_user_data(struct qt_xcomposite *qt_xcomposite, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) qt_xcomposite, user_data);
}

static inline void *
qt_xcomposite_get_user_data(struct qt_xcomposite *qt_xcomposite)
{
	return wl_proxy_get_user_data((struct wl_proxy *) qt_xcomposite);
}

static inline void
qt_xcomposite_destroy(struct qt_xcomposite *qt_xcomposite)
{
	wl_proxy_destroy((struct wl_proxy *) qt_xcomposite);
}

static inline struct wl_buffer *
qt_xcomposite_create_buffer(struct qt_xcomposite *qt_xcomposite, uint32_t x_window, int32_t width, int32_t height)
{
	struct wl_proxy *id;

	id = wl_proxy_marshal_constructor((struct wl_proxy *) qt_xcomposite,
			 QT_XCOMPOSITE_CREATE_BUFFER, &wl_buffer_interface, NULL, x_window, width, height);

	return (struct wl_buffer *) id;
}

#ifdef  __cplusplus
}
#endif

#endif
