#include "qwayland-server-drm-egl-server-buffer.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_drm_egl_server_buffer::qt_drm_egl_server_buffer(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_drm_egl_server_buffer::qt_drm_egl_server_buffer(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_drm_egl_server_buffer::qt_drm_egl_server_buffer()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_drm_egl_server_buffer::~qt_drm_egl_server_buffer()
    {
    }

    void qt_drm_egl_server_buffer::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_drm_egl_server_buffer::Resource *qt_drm_egl_server_buffer::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_drm_egl_server_buffer::Resource *qt_drm_egl_server_buffer::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_drm_egl_server_buffer::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_drm_egl_server_buffer_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_drm_egl_server_buffer::Resource *qt_drm_egl_server_buffer::drm_egl_server_buffer_allocate()
    {
        return new Resource;
    }

    void qt_drm_egl_server_buffer::drm_egl_server_buffer_bind_resource(Resource *)
    {
    }

    void qt_drm_egl_server_buffer::drm_egl_server_buffer_destroy_resource(Resource *)
    {
    }

    void qt_drm_egl_server_buffer::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_drm_egl_server_buffer *that = static_cast<qt_drm_egl_server_buffer *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_drm_egl_server_buffer::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_drm_egl_server_buffer *that = resource->drm_egl_server_buffer_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->drm_egl_server_buffer_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_drm_egl_server_buffer::Resource *qt_drm_egl_server_buffer::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = drm_egl_server_buffer_allocate();
        resource->drm_egl_server_buffer_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_drm_egl_server_buffer_interface, version, id);
        wl_resource_set_implementation(handle, 0, resource, destroy_func);
        resource->handle = handle;
        drm_egl_server_buffer_bind_resource(resource);
        return resource;
    }

    void qt_drm_egl_server_buffer::send_server_buffer_created(struct ::wl_resource *id, int32_t name, int32_t width, int32_t height, int32_t stride, int32_t format)
    {
        send_server_buffer_created(
            m_resource->handle,
            id,
            name,
            width,
            height,
            stride,
            format);
    }

    void qt_drm_egl_server_buffer::send_server_buffer_created(struct ::wl_resource *resource, struct ::wl_resource *id, int32_t name, int32_t width, int32_t height, int32_t stride, int32_t format)
    {
        qt_drm_egl_server_buffer_send_server_buffer_created(
            resource,
            id,
            name,
            width,
            height,
            stride,
            format);
    }

}

QT_END_NAMESPACE
