#ifndef QT_WAYLAND_SERVER_DRM_EGL_SERVER_BUFFER
#define QT_WAYLAND_SERVER_DRM_EGL_SERVER_BUFFER

#include "wayland-server.h"
#include "wayland-drm-egl-server-buffer-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  qt_drm_egl_server_buffer
    {
    public:
        qt_drm_egl_server_buffer(struct ::wl_client *client, int id, int version);
        qt_drm_egl_server_buffer(struct ::wl_display *display, int version);
        qt_drm_egl_server_buffer();

        virtual ~qt_drm_egl_server_buffer();

        class Resource
        {
        public:
            Resource() : drm_egl_server_buffer_object(0), handle(0) {}
            virtual ~Resource() {}

            qt_drm_egl_server_buffer *drm_egl_server_buffer_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        enum format {
            format_RGBA32 = 0,
            format_A8 = 1
        };

        void send_server_buffer_created(struct ::wl_resource *id, int32_t name, int32_t width, int32_t height, int32_t stride, int32_t format);
        void send_server_buffer_created(struct ::wl_resource *resource, struct ::wl_resource *id, int32_t name, int32_t width, int32_t height, int32_t stride, int32_t format);

    protected:
        virtual Resource *drm_egl_server_buffer_allocate();

        virtual void drm_egl_server_buffer_bind_resource(Resource *resource);
        virtual void drm_egl_server_buffer_destroy_resource(Resource *resource);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
