#include "qwayland-server-xcomposite.h"

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    qt_xcomposite::qt_xcomposite(struct ::wl_client *client, int id, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(client, id, version);
    }

    qt_xcomposite::qt_xcomposite(struct ::wl_display *display, int version)
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
        init(display, version);
    }

    qt_xcomposite::qt_xcomposite()
        : m_resource_map()
        , m_resource(0)
        , m_global(0)
    {
    }

    qt_xcomposite::~qt_xcomposite()
    {
    }

    void qt_xcomposite::init(struct ::wl_client *client, int id, int version)
    {
        m_resource = bind(client, id, version);
    }

    qt_xcomposite::Resource *qt_xcomposite::add(struct ::wl_client *client, int version)
    {
        Resource *resource = bind(client, 0, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    qt_xcomposite::Resource *qt_xcomposite::add(struct ::wl_client *client, int id, int version)
    {
        Resource *resource = bind(client, id, version);
        m_resource_map.insert(client, resource);
        return resource;
    }

    void qt_xcomposite::init(struct ::wl_display *display, int version)
    {
        m_global = wl_global_create(display, &::qt_xcomposite_interface, version, this, bind_func);
        m_globalVersion = version;
    }

    qt_xcomposite::Resource *qt_xcomposite::xcomposite_allocate()
    {
        return new Resource;
    }

    void qt_xcomposite::xcomposite_bind_resource(Resource *)
    {
    }

    void qt_xcomposite::xcomposite_destroy_resource(Resource *)
    {
    }

    void qt_xcomposite::bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id)
    {
        qt_xcomposite *that = static_cast<qt_xcomposite *>(data);
        that->add(client, id, qMin(that->m_globalVersion, version));
    }

    void qt_xcomposite::destroy_func(struct ::wl_resource *client_resource)
    {
        Resource *resource = Resource::fromResource(client_resource);
        qt_xcomposite *that = resource->xcomposite_object;
        that->m_resource_map.remove(resource->client(), resource);
        that->xcomposite_destroy_resource(resource);
        delete resource;
#if !WAYLAND_VERSION_CHECK(1, 2, 0)
        free(client_resource);
#endif
    }

    qt_xcomposite::Resource *qt_xcomposite::bind(struct ::wl_client *client, uint32_t id, int version)
    {
        Resource *resource = xcomposite_allocate();
        resource->xcomposite_object = this;

        struct ::wl_resource *handle = wl_resource_create(client, &::qt_xcomposite_interface, version, id);
        wl_resource_set_implementation(handle, &m_qt_xcomposite_interface, resource, destroy_func);
        resource->handle = handle;
        xcomposite_bind_resource(resource);
        return resource;
    }

    const struct ::qt_xcomposite_interface qt_xcomposite::m_qt_xcomposite_interface = {
        qt_xcomposite::handle_create_buffer
    };

    void qt_xcomposite::xcomposite_create_buffer(Resource *, uint32_t, uint32_t , int32_t , int32_t )
    {
    }


    void qt_xcomposite::handle_create_buffer(
        ::wl_client *client,
        struct wl_resource *resource,
        uint32_t id,
        uint32_t x_window,
        int32_t width,
        int32_t height)
    {
        Q_UNUSED(client);
        Resource *r = Resource::fromResource(resource);
        static_cast<qt_xcomposite *>(r->xcomposite_object)->xcomposite_create_buffer(
            r,
            id,
            x_window,
            width,
            height);
    }

    void qt_xcomposite::send_root(const QString &display_name, uint32_t root_window)
    {
        send_root(
            m_resource->handle,
            display_name,
            root_window);
    }

    void qt_xcomposite::send_root(struct ::wl_resource *resource, const QString &display_name, uint32_t root_window)
    {
        qt_xcomposite_send_root(
            resource,
            display_name.toUtf8().constData(),
            root_window);
    }

}

QT_END_NAMESPACE
