#ifndef QT_WAYLAND_SERVER_XCOMPOSITE
#define QT_WAYLAND_SERVER_XCOMPOSITE

#include "wayland-server.h"
#include "wayland-xcomposite-server-protocol.h"
#include <QByteArray>
#include <QMultiMap>
#include <QString>

#ifndef WAYLAND_VERSION_CHECK
#define WAYLAND_VERSION_CHECK(major, minor, micro) \
    ((WAYLAND_VERSION_MAJOR > (major)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR > (minor)) || \
    (WAYLAND_VERSION_MAJOR == (major) && WAYLAND_VERSION_MINOR == (minor) && WAYLAND_VERSION_MICRO >= (micro)))
#endif

QT_BEGIN_NAMESPACE

namespace QtWaylandServer {
    class  qt_xcomposite
    {
    public:
        qt_xcomposite(struct ::wl_client *client, int id, int version);
        qt_xcomposite(struct ::wl_display *display, int version);
        qt_xcomposite();

        virtual ~qt_xcomposite();

        class Resource
        {
        public:
            Resource() : xcomposite_object(0), handle(0) {}
            virtual ~Resource() {}

            qt_xcomposite *xcomposite_object;
            struct ::wl_resource *handle;

            struct ::wl_client *client() const { return handle->client; }
            int version() const { return wl_resource_get_version(handle); }

            static Resource *fromResource(struct ::wl_resource *resource) { return static_cast<Resource *>(resource->data); }
        };

        void init(struct ::wl_client *client, int id, int version);
        void init(struct ::wl_display *display, int version);

        Resource *add(struct ::wl_client *client, int version);
        Resource *add(struct ::wl_client *client, int id, int version);
        Resource *add(struct wl_list *resource_list, struct ::wl_client *client, int id, int version);

        Resource *resource() { return m_resource; }
        const Resource *resource() const { return m_resource; }

        QMultiMap<struct ::wl_client*, Resource*> resourceMap() { return m_resource_map; }
        const QMultiMap<struct ::wl_client*, Resource*> resourceMap() const { return m_resource_map; }

        bool isGlobal() const { return m_global != 0; }
        bool isResource() const { return m_resource != 0; }

        void send_root(const QString &display_name, uint32_t root_window);
        void send_root(struct ::wl_resource *resource, const QString &display_name, uint32_t root_window);

    protected:
        virtual Resource *xcomposite_allocate();

        virtual void xcomposite_bind_resource(Resource *resource);
        virtual void xcomposite_destroy_resource(Resource *resource);

        virtual void xcomposite_create_buffer(Resource *resource, uint32_t id, uint32_t x_window, int32_t width, int32_t height);

    private:
        static void bind_func(struct ::wl_client *client, void *data, uint32_t version, uint32_t id);
        static void destroy_func(struct ::wl_resource *client_resource);

        Resource *bind(struct ::wl_client *client, uint32_t id, int version);

        static const struct ::qt_xcomposite_interface m_qt_xcomposite_interface;

        static void handle_create_buffer(
            ::wl_client *client,
            struct wl_resource *resource,
            uint32_t id,
            uint32_t x_window,
            int32_t width,
            int32_t height);

        QMultiMap<struct ::wl_client*, Resource*> m_resource_map;
        Resource *m_resource;
        struct ::wl_global *m_global;
        uint32_t m_globalVersion;
    };
}

QT_END_NAMESPACE

#endif
